/**
 * 
 */
package org.zengsource.util;

import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;

/**
 * Java 2D utilities.
 * 
 * @author snzeng
 * @since 6.0
 */
public abstract class GraphicUtil {

	public static Font createFont(File ttfFile) throws FontFormatException, IOException {
		return Font.createFont(Font.TRUETYPE_FONT, ttfFile);
	}

	public static ImageReader getReader(String formatName) {
		return ImageIO.getImageReadersByFormatName(formatName).next();
	}

	public static ImageWriter getWriter(String formatName) {
		return ImageIO.getImageWritersByFormatName(formatName).next();
	}

	public static BufferedImage read(File image) {
		String imageName = image.getName();
		ImageReader reader = getReader(imageName.substring(imageName.lastIndexOf(".") + 1));
		ImageInputStream iis = null;
		int imageIndex = 0;
		try {
			iis = ImageIO.createImageInputStream(image);
			reader.setInput(iis, true);
			return reader.read(imageIndex);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	public static void write(BufferedImage bi, File output) {
		String outputName = output.getName();
		ImageWriter writer = getWriter(outputName.substring(outputName.lastIndexOf(".") + 1));
		ImageOutputStream ios = null;
		try {
			ios = ImageIO.createImageOutputStream(output);
			writer.setOutput(ios);
			writer.write(bi);
			ios.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
