/**
 * 
 */
package org.zengsource.util;

import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;

/**
 * Java 2D utilities.
 * 
 * @author snzeng
 * @since 6.0
 */
public abstract class GraphicsUtil {

	public static Font createFont(File ttfFile) throws FontFormatException, IOException {
		return Font.createFont(Font.TRUETYPE_FONT, ttfFile);
	}

	public static ImageReader getReader(String formatName) {
		return ImageIO.getImageReadersByFormatName(formatName).next();
	}

	public static ImageWriter getWriter(String formatName) {
		return ImageIO.getImageWritersByFormatName(formatName).next();
	}

	public static BufferedImage read(File image) {
		String imageName = image.getName();
		ImageReader reader = getReader(imageName.substring(imageName.lastIndexOf(".") + 1));
		ImageInputStream iis = null;
		int imageIndex = 0;
		try {
			iis = ImageIO.createImageInputStream(image);
			reader.setInput(iis, true);
			return reader.read(imageIndex);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	public static void write(BufferedImage bi, File output) {
		String outputName = output.getName();
		ImageWriter writer = getWriter(outputName.substring(outputName.lastIndexOf(".") + 1));
		ImageOutputStream ios = null;
		try {
			ios = ImageIO.createImageOutputStream(output);
			writer.setOutput(ios);
			writer.write(bi);
			ios.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void combine(String path, BufferedImage bi1, BufferedImage bi2) {
		Graphics2D g2d = (Graphics2D) bi1.getGraphics();
		int x = bi1.getWidth() / 2;
		int y = bi2.getHeight() / 2;
		g2d.drawImage(bi2, x, y, null);

		write(bi1, new File(path));
	}

	public static void main(String[] args) {
		// write(read("input/My Computer2.png"), "input/My Computer3.png");
		BufferedImage bi1 = read(new File("input/My Computer.png"));
		BufferedImage bi2 = read(new File("input/XTRD-arrow.png"));
		combine("output/My Computer3.png", bi1, bi2);
	}

}
