/**
 * 
 */
package org.zengsource.util;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * @author zeng.xiaoning
 * 
 */
public abstract class StringUtil {

	public static boolean isBlank(String str) {
		return str == null || str.trim().length() == 0;
	}

	public static boolean notBlank(String str) {
		return !isBlank(str);
	}

	public static boolean isEmail(String str) {
		if (isBlank(str))
			return false;
		String regex = "^([a-z0-9A-Z]+[-|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$";
		return Pattern.matches(regex, str);
	}

	public static String random(int len) {
		String letter = "ABCDEFGHIJKLMNOPQRSTabcdefghijklmnopqrst1234567890";
		// System.out.println(letter.length());
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < len; i++) {
			sb.append(letter.charAt(NumberUtil.random(50)));
		}
		return sb.toString();
	}

	/** 截取结束字符之前的部分。*/
	public static String substring(String str, String end) {
		int pos = str.indexOf(end);
		return str.substring(0, pos);
	}

	/** 截取开始字符之后的部分。*/
	public static String substring2(String str, String start) {
		int pos = str.lastIndexOf(start);
		return str.substring(pos + 1, str.length());
	}

	public static Map<String, String> parseQuery(String url) {
		if (isBlank(url)) {
			return null;
		}
		int pos = url.indexOf("?");
		if (pos > -1) {
			url = url.substring(pos + 1);
		}
		String[] parts = url.split("&");
		if (parts == null) {
			return null;
		}
		Map<String, String> map = new HashMap<String, String>();
		for (String part : parts) {
			String[] keyValue = part.split("=");
			if (keyValue != null) {
				if (keyValue.length == 1) {
					map.put(keyValue[0], "");
				} else if (keyValue.length == 2) {
					map.put(keyValue[0], keyValue[1]);
				} else {
					continue;
				}
			}
		}
		return map;
	}

	public static void main(String[] args) {
		String email = "188@qq.com";
		System.out.println(StringUtil.isEmail(email));

		System.out.println(StringUtil.random(16));
		System.out.println(StringUtil.substring("abc@gmail.com", "@"));
		System.out.println(StringUtil.substring2("avatar.png", "."));
		System.out.println(StringUtil.parseQuery("html?abc=123&def&hij=abc"));
	}

}
