/**
 * 
 */
package org.zengsource.util.spring.dao.orm;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.zengsource.util.IDUtil;
import org.zengsource.util.StringUtil;
import org.zengsource.util.spring.dao.DaoInterface;

/**
 * @author zeng.xiaoning
 * @since 6.0
 */
public abstract class Hibernate4DaoTemplate implements DaoInterface {

	// ~ 对象属性 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	protected Logger logger = Logger.getLogger(getClass());
	protected HibernateTemplate hibernateTemplate;

	// ~ 构造方法 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	public Hibernate4DaoTemplate() {
	}

	// ~ 逻辑方法 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	public abstract Class<?> getPrototypeClass();

	/** 重载此方法实现模糊查找。 */
	protected Criterion[] queryCriterions(String q) {
		return new Criterion[] {};
	}

	protected String uniqueId() {
		return IDUtil.generate();
	}

	/**
	 * Update: 支持自增 ID
	 * 
	 * @see net.zeng.spring.dao.DaoInterface#save(java.lang.Object)
	 */
	public void save(Object object) {
		String idType = null;
		try {
			Method idGetter = object.getClass().getMethod("getId");
			idType = idGetter.getGenericReturnType().toString();
			if ("class java.lang.String".equals(idType)) {
				String id = BeanUtils.getSimpleProperty(object, "id");
				if (StringUtil.isBlank(id)) {
					id = uniqueId();
					BeanUtils.setProperty(object, "id", id);
					this.hibernateTemplate.save(object);
				} else {
					this.hibernateTemplate.saveOrUpdate(object);
				}
			} else if ("int,long".contains(idType) || "class java.lang.Integer".equals(idType)
					|| "class java.lang.Long".equals(idType)) {
				Integer id = 0;
				try {
					id = Integer.parseInt(BeanUtils.getSimpleProperty(object, "id"));
				} catch (NumberFormatException e) {
					// e.printStackTrace();
					logger.debug("save()-> Null id.");
				}
				if (id == 0) {
					this.hibernateTemplate.save(object);
				} else {
					this.hibernateTemplate.saveOrUpdate(object);
				}
			}
		} catch (SecurityException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (NoSuchMethodException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
		}
		if (idType == null) {// 直接保存
			this.hibernateTemplate.saveOrUpdate(object);
		}
	}

	/** 插入记录修改和创建时间。 */
	public void updateTime(Object entity, String createdTimeStr, String updatedTimeStr) {
		if (StringUtil.isBlank(createdTimeStr)) {
			createdTimeStr = "createdTime";
		}
		if (StringUtil.isBlank(updatedTimeStr)) {
			updatedTimeStr = "updatedTime";
		}
		Date now = new Date();
		try { // Update createdTime
			Object createdTime = BeanUtils.getProperty(entity, createdTimeStr);
			if (createdTime == null) {
				BeanUtils.setProperty(entity, createdTimeStr, now);
			}
		} catch (IllegalAccessException e) {
			// e.printStackTrace();
		} catch (InvocationTargetException e) {
			// e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// e.printStackTrace();
		} finally {
			// Ignore it ...
		}
		try { // Update updatedTime
			BeanUtils.setProperty(entity, updatedTimeStr, now);
		} catch (IllegalAccessException e) {
			// e.printStackTrace();
		} catch (InvocationTargetException e) {
			// e.printStackTrace();
		} finally {
			// Ignore it ...
		}
	}

	/** 使用默认属性名：createdTime 和 updatedTime。 */
	public void updateTime(Object entity) {
		this.updateTime(entity, null, null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.zeng.spring.dao.DaoInterface#queryById(java.lang.Object)
	 */
	public Object queryById(final Object id) {
		return this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				criteria.add(Restrictions.eq("id", id));
				return criteria.uniqueResult();
			}
		});
	}

	public Integer queryCount(String q) {
		if (StringUtil.isBlank(q)) {
			return this.queryCount(new Criterion[]{});
		}
		return this.queryCount(this.queryCriterions(q));
	}

	public Integer queryCount(Map<String, Object> criteraMap) {
		return this.queryCountL(criteraMap).intValue();
	}

	public Long queryCountL(final Map<String, Object> criteraMap) {
		Object result = this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				buildCritera(criteria, criteraMap);
				criteria.setProjection(Projections.count("id"));
				return criteria.uniqueResult();
			}
		});
		return result == null ? 0 : (Long) result;
	}

	public Integer queryCount(Criterion[] criterions) {
		return this.queryCountL(criterions).intValue();
	}

	public Long queryCountL(final Criterion[] criterions) {
		Object result = this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criterions != null) {
					for (Criterion cr : criterions) {
						criteria.add(cr);
					}
				}
				criteria.setProjection(Projections.count("id"));
				return criteria.uniqueResult();
			}
		});
		try {
			return result == null ? 0 : (Long) result;
		} catch (ClassCastException e) {
			return result == null ? 0 : ((Integer) result).longValue();
		}
	}

	public List<?> query(String q, Integer start, Integer limit) {
		if (StringUtil.isBlank(q)) {
			return this.query(new Criterion[0], start, limit);
		}
		return this.query(this.queryCriterions(q), start, limit);
	}

	public List<?> query(final Map<String, Object> criteriaMap, final Integer start,
			final Integer limit) {
		return this.hibernateTemplate.executeFind(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criteriaMap != null) {
					buildCritera(criteria, criteriaMap);
				}
				if (start != null && start > 0) {
					criteria.setFirstResult(start);
				}
				if (limit != null && limit > 0) {
					criteria.setMaxResults(limit);
				}
				return criteria.list();
			}
		});
	}

	public List<?> query(final Criterion[] criterions, final Integer start, final Integer limit) {
		return this.query(criterions, start, limit, null);
	}

	public List<?> query(final Criterion[] criterions, final Integer start, final Integer limit,
			final Order order) {
		return this.hibernateTemplate.executeFind(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criterions != null && criterions.length > 0) {
					for (Criterion cr : criterions) {
						criteria.add(cr);
					}
				}
				if (start != null && start > 0) {
					criteria.setFirstResult(start);
				}
				if (limit != null && limit > 0) {
					criteria.setMaxResults(limit);
				}
				if (order != null) {
					criteria.addOrder(order);
				}
				return criteria.list();
			}
		});
	}

	public Object queryUnique(final Map<String, Object> criteriaMap) {
		return this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				buildCritera(criteria, criteriaMap);
				return criteria.uniqueResult();
			}
		});
	}

	public Object queryUnique(final Criterion[] criterions) {
		return this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criterions != null) {
					for (Criterion cr : criterions) {
						criteria.add(cr);
					}
				}
				return criteria.uniqueResult();
			}
		});
	}

	public Object queryUnique(String key, Object value) {
		Map<String, Object> criteriaMap = new HashMap<String, Object>();
		criteriaMap.put(key, value);
		return queryUnique(criteriaMap);
	}

	private void buildCritera(Criteria criteria, Map<String, Object> criteriaMap) {
		for (String key : criteriaMap.keySet()) {
			Object value = criteriaMap.get(key);
			if (value == null) {
				criteria.add(Restrictions.isNull(key));
			} else if (value instanceof LikeMatcher) {
				LikeMatcher likeMatcher = (LikeMatcher) value;
				criteria.add(Restrictions.like(key, likeMatcher.getValue(), likeMatcher.getMode()));
			} else if (value instanceof SQLRestriction) {
				SQLRestriction sqlRt = (SQLRestriction) value;
				criteria.add(Restrictions.sqlRestriction(key, sqlRt.getValues(), sqlRt.getTypes()));
			} else {
				criteria.add(Restrictions.eq(key, value));
			}
		}
	}

	// ~ g^setX ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	public SessionFactory getSessionFactory() {
		return this.hibernateTemplate.getSessionFactory();
	}

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.hibernateTemplate = new HibernateTemplate(sessionFactory);
	}
}
