/**
 * 
 */
package org.zengsource.util.spring.dao.orm;

import java.lang.reflect.InvocationTargetException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Projections;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.zengsource.util.IDUtil;
import org.zengsource.util.StringUtil;
import org.zengsource.util.spring.dao.DaoInterface;


/**
 * @author zeng.xiaoning
 * @since 6.0
 */
public abstract class HibernateDaoTemplate implements DaoInterface {

	// ~ 对象属性 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	protected Logger logger = Logger.getLogger(getClass());
	protected HibernateTemplate hibernateTemplate;

	// ~ 构造方法 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	public HibernateDaoTemplate() {
	}

	// ~ 逻辑方法 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	public abstract Class<?> getPrototypeClass();

	protected String uniqueId() {
		return IDUtil.generate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.zeng.spring.dao.DaoInterface#save(java.lang.Object)
	 */
	public void save(Object object) {
		String id = null;
		try {
			id = BeanUtils.getSimpleProperty(object, "id");
			if (StringUtil.isBlank(id)) {
				id = uniqueId();
				BeanUtils.setProperty(object, "id", id);
				this.hibernateTemplate.save(object);
			} else {
				this.hibernateTemplate.saveOrUpdate(object);
			}
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.zeng.spring.dao.DaoInterface#queryById(java.lang.Object)
	 */
	public Object queryById(final Object id) {
		return this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				criteria.add(Expression.eq("id", id));
				return criteria.uniqueResult();
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.zeng.spring.dao.DaoInterface#queryCount(java.util.Map)
	 */
	public Integer queryCount(Map<String, Object> criteraMap) {
		return this.queryCountL(criteraMap).intValue();
	}
	
	public Long queryCountL(final Map<String, Object> criteraMap) {
		Object result = this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				buildCritera(criteria, criteraMap);
				criteria.setProjection(Projections.count("id"));
				return criteria.uniqueResult();
			}
		});
		return result == null ? 0 : (Long) result;
	}

	public Integer queryCount( Criterion[] criterions) {
		return this.queryCountL(criterions).intValue(); 
	}
	
	public Long queryCountL(final Criterion[] criterions) {
		Object result = this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criterions != null) {
					for (Criterion cr : criterions) {
						criteria.add(cr);
					}
				}
				criteria.setProjection(Projections.count("id"));
				return criteria.uniqueResult();
			}
		});
		try {
			return result == null ? 0 : (Long) result;
		} catch (ClassCastException e) {
			return result == null ? 0 : ((Integer) result).longValue();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.zeng.spring.dao.DaoInterface#query(java.util.Map,
	 * java.lang.Integer, java.lang.Integer)
	 */
	public List<?> query(final Map<String, Object> criteriaMap, final Integer start,
			final Integer limit) {
		return this.hibernateTemplate.executeFind(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criteriaMap != null) {
					buildCritera(criteria, criteriaMap);
				}
				if (start != null && start > 0) {
					criteria.setFirstResult(start);
				}
				if (limit != null && limit > 0) {
					criteria.setMaxResults(limit);
				}
				return criteria.list();
			}
		});
	}

	public List<?> query(final Criterion[] criterions, final Integer start, final Integer limit) {
		return this.hibernateTemplate.executeFind(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criterions != null && criterions.length > 0) {
					for (Criterion cr : criterions) {
						criteria.add(cr);
					}
				}
				if (start != null && start > 0) {
					criteria.setFirstResult(start);
				}
				if (limit != null && limit > 0) {
					criteria.setMaxResults(limit);
				}
				return criteria.list();
			}
		});
	}

	public Object queryUnique(final Map<String, Object> criteriaMap) {
		return this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				buildCritera(criteria, criteriaMap);
				return criteria.uniqueResult();
			}
		});
	}

	public Object queryUnique(final Criterion[] criterions) {
		return this.hibernateTemplate.execute(new HibernateCallback<Object>() {
			public Object doInHibernate(Session session) throws HibernateException, SQLException {
				Criteria criteria = session.createCriteria(getPrototypeClass());
				if (criterions != null) {
					for (Criterion cr : criterions) {
						criteria.add(cr);
					}
				}
				return criteria.uniqueResult();
			}
		});
	}

	public Object queryUnique(String key, Object value) {
		Map<String, Object> criteriaMap = new HashMap<String, Object>();
		criteriaMap.put(key, value);
		return queryUnique(criteriaMap);
	}

	private void buildCritera(Criteria criteria, Map<String, Object> criteriaMap) {
		for (String key : criteriaMap.keySet()) {
			Object value = criteriaMap.get(key);
			if (value == null) {
				criteria.add(Expression.isNull(key));
			} else if (value instanceof LikeMatcher) {
				LikeMatcher likeMatcher = (LikeMatcher) value;
				criteria.add(Expression.like(key, likeMatcher.getValue(), likeMatcher.getMode()));
			} else if (value instanceof SQLRestriction) {
				SQLRestriction sqlRt = (SQLRestriction) value;
				criteria.add(Expression.sqlRestriction(key, sqlRt.getValues(), sqlRt.getTypes()));
			} else {
				criteria.add(Expression.eq(key, value));
			}
		}
	}

	// ~ g^setX ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //

	public SessionFactory getSessionFactory() {
		return this.hibernateTemplate.getSessionFactory();
	}

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.hibernateTemplate = new HibernateTemplate(sessionFactory);
	}
}
